/* gcompris - clickgame.c
 *
 * Time-stamp: <2001/03/26 01:14:09 bruno>
 *
 * Copyright (C) 2000 Bruno Coudoin
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * The bar button common to each games
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <dirent.h>
#include <gnome.h>

#include "gcompris.h"
#include "gcompris-board.h"
#include "properties.h"
#include "gameutil.h"

#define SOUNDLISTFILE PACKAGE

#define BAR_GAP 30		/* Value used to fill space above and under icons in the bar */

static gint item_event_bar(GnomeCanvasItem *item, GdkEvent *event, gpointer data);

static gint current_level;
static GnomeCanvasItem *level_item = NULL;
static guint level_handler_id;
static gfloat maxtimer;

GtkWidget *statusbar= NULL;

/*
 * Main entry point 
 * ----------------
 *
 */


/*
 * Do all the bar display and register the events
 */
void bar_start (GnomeCanvas *theCanvas, GtkWidget *theStatusbar)
{

  GdkImlibImage   *pixmap = NULL;
  GnomeCanvasItem *item;
  GnomeCanvasItem *rootitem, *rectitem;
  gint16           width, height;
  double           zoom;

  statusbar = theStatusbar;

  width  = BOARDWIDTH-1;
  height = BARHEIGHT-2;

  rootitem = \
    gnome_canvas_item_new (gnome_canvas_root(theCanvas),
			   gnome_canvas_group_get_type (),
			   "x", (double)0,
			   "y", (double)0,
			   NULL);

  rectitem = \
    gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
			   gnome_canvas_rect_get_type (),
			   "x1", (double)0,
			   "y1", (double)0,
			   "x2", (double)width,
			   "y2", (double)height,
			   "fill_color", "black",
			   "outline_color", "red",
			   "width_units", (double)3,
			   NULL);


  pixmap = load_pixmap("buttons/bar_bg.jpg");
  item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
				gnome_canvas_image_get_type (),
				"image", pixmap, 
				"x", (double) 3,
				"y", (double) 3,
				"width", (double)  pixmap->rgb_width-6,
				"height", (double) pixmap->rgb_height-6,
				"anchor", GTK_ANCHOR_NW,
				NULL);

  pixmap = load_button_pixmap("home");
  zoom = (double)(height-BAR_GAP)/(double)pixmap->rgb_height;
  item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
				gnome_canvas_image_get_type (),
				"image", pixmap, 
				"x", (double) width*0.9,
				"y", (double) (height-pixmap->rgb_height*zoom)/2,
				"width", (double)  pixmap->rgb_width*zoom,
				"height", (double) pixmap->rgb_height*zoom,
				"anchor", GTK_ANCHOR_NW,
				NULL);

  gtk_signal_connect(GTK_OBJECT(item), "event",
		     (GtkSignalFunc) item_event_bar,
		     "home");
  set_image_focus(pixmap, FALSE);
  gtk_signal_connect(GTK_OBJECT(item), "event",
		     (GtkSignalFunc) item_event_focus,
		     pixmap);


  pixmap = load_button_pixmap("ok");
  zoom = (double)(height-BAR_GAP)/(double)pixmap->rgb_height;
  item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
				gnome_canvas_image_get_type (),
				"image", pixmap, 
				"x", (double) width*0.7,
				"y", (double) (height-pixmap->rgb_height*zoom)/2,
				"width", (double)  pixmap->rgb_width*zoom,
				"height", (double) pixmap->rgb_height*zoom,
				"anchor", GTK_ANCHOR_NW,
				NULL);

  gtk_signal_connect(GTK_OBJECT(item), "event",
		     (GtkSignalFunc) item_event_bar,
		     "ok");
  set_image_focus(pixmap, FALSE);
  gtk_signal_connect(GTK_OBJECT(item), "event",
		     (GtkSignalFunc) item_event_focus,
		     pixmap);


  pixmap = load_pixmap("dice/gnome-dice1.png");
  zoom = (double)(height-BAR_GAP)/(double)pixmap->rgb_height;
  level_item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
				      gnome_canvas_image_get_type (),
				      "image", pixmap, 
				      "x", (double) width*0.5,
				      "y", (double) (height-pixmap->rgb_height*zoom)/2,
				      "width", (double)  pixmap->rgb_width*zoom,
				      "height", (double) pixmap->rgb_height*zoom,
				      "anchor", GTK_ANCHOR_NW,
				      NULL);
  
  current_level = 1;
  gtk_signal_connect(GTK_OBJECT(level_item), "event",
		     (GtkSignalFunc) item_event_bar,
		     "level");
  set_image_focus(pixmap, FALSE);
  level_handler_id =   gtk_signal_connect(GTK_OBJECT(level_item), "event",
					  (GtkSignalFunc) item_event_focus,
					  pixmap);

}


void bar_set_level(GcomprisGame *gcomprisGame)
{
  char *str = NULL;
  GdkImlibImage *pixmap = NULL;
      
  /* Non yet initialized : Something Wrong */
  if(level_item==NULL)
    {
      printf("ERROR : in bar_set_level, level_item uninitialized : should not happen\n");
      return;
    }

  if(gcomprisGame!=NULL)
    {
      str = g_strdup_printf("dice/gnome-dice%d.png", gcomprisGame->level);
      pixmap = load_pixmap(str);
      g_free(str);
      gnome_canvas_item_set (level_item,
			     "image", pixmap,
			     NULL);
      /* Warning, I have to reconnect to highlight the new image */
      gtk_signal_disconnect	(GTK_OBJECT(level_item), level_handler_id);
      level_handler_id = gtk_signal_connect(GTK_OBJECT(level_item), "event",
					    (GtkSignalFunc) item_event_focus,
					    pixmap);
    }
  
  current_level=gcomprisGame->level;
}



void
bar_set_timer (guint time)
{
  gnome_appbar_set_progress (GNOME_APPBAR (statusbar), (gfloat) time/maxtimer);
}


void
bar_set_maxtimer (guint time)
{
  maxtimer= time;
  gnome_appbar_set_progress (GNOME_APPBAR (statusbar), 0.0);
}


/* Setting text line information.  */
void
bar_set_message (const gchar *message)
{
  gnome_appbar_set_status (GNOME_APPBAR (statusbar), message);
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/



/* Callback for the bar operations */
static gint
item_event_bar(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
{
  GcomprisGame *gcomprisGame = get_running_game();

  switch (event->type) 
    {
    case GDK_ENTER_NOTIFY:
      break;
    case GDK_LEAVE_NOTIFY:
      break;
    case GDK_BUTTON_PRESS:
      if(!strcmp((char *)data, "ok"))
	{
	  if(gcomprisGame!=NULL)
	    {
	      if(gcomprisGame->ok != NULL)
		{
		  gcomprisGame->ok();
		}
	    }
	}
      else if(!strcmp((char *)data, "level"))
	{
	  play_sound (SOUNDLISTFILE, "gobble");
	  current_level++;
	  if(current_level>gcomprisGame->maxlevel)
	    current_level=1;
	  
	  if(gcomprisGame->set_level != NULL)
	    {
	      gcomprisGame->set_level(current_level);
	    }
	}
      else if(!strcmp((char *)data, "home"))
	{
	  play_sound (SOUNDLISTFILE, "gobble");
	  end_game();
	}
      break;
      
    default:
      break;
    }
  return FALSE;

}




/* Local Variables: */
/* mode:c */
/* eval:(load-library "time-stamp") */
/* eval:(make-local-variable 'write-file-hooks) */
/* eval:(add-hook 'write-file-hooks 'time-stamp) */
/* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
/* End: */
