/* gcompris - gameutil.c
 *
 * Time-stamp: <2000/07/16 01:36:49 bruno>
 *
 * Copyright (C) 2000 Bruno Coudoin
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "gcompris.h"
#include "gameutil.h"

#define IMAGEEXTENSION ".png"

/* default gnome pixmap directory in which this game tales the icon */
static char *lettersdir = "letters/";
static char *buttonsdir = "buttons/";

void set_game_size (GnomeCanvas *theCanvas, GcomprisGame *gcomprisGame)
{

  /* Fixed since I use the canvas own pixel_per_unit scheme */
  gcomprisGame->width  = BOARDWIDTH;
  gcomprisGame->height = BOARDHEIGHT;

}


GdkImlibImage *load_button_pixmap(char *operation)
{
  gchar *filename;
  GdkImlibImage *pixmap;

  filename = g_strdup_printf("%s/%s%s%s", PACKAGE_PIXMAPS_DIR, buttonsdir, operation, IMAGEEXTENSION);

  if (!g_file_exists (filename)) {
    g_error (_("Couldn't find file %s !"), filename);
  }

  pixmap = gdk_imlib_load_image (filename);
  
  g_free (filename);
  
  return(pixmap);
}

GdkImlibImage *load_operation_pixmap(char operation)
{
  gchar *filename;
  GdkImlibImage *pixmap;

  filename = g_strdup_printf("%s/%s%c%s", PACKAGE_PIXMAPS_DIR, lettersdir, operation, IMAGEEXTENSION);

  if (!g_file_exists (filename)) {
    g_error (_("Couldn't find file %s !"), filename);
  }

  pixmap = gdk_imlib_load_image (filename);
  
  g_free (filename);
  
  return(pixmap);
}

GdkImlibImage *load_number_pixmap(char number)
{
  gchar *filename;
  GdkImlibImage *pixmap;

  filename = g_strdup_printf("%s/%s%c%s", PACKAGE_PIXMAPS_DIR, lettersdir, number, IMAGEEXTENSION);

  if (!g_file_exists (filename)) {
    g_error (_("Couldn't find file %s !"), filename);
  }

  pixmap = gdk_imlib_load_image (filename);
  
  g_free (filename);
  
  return(pixmap);
}


GdkImlibImage *load_pixmap(char *pixmapfile)
{
  gchar *filename;
  GdkImlibImage *smallnumbers_pixmap;

  filename = g_strdup_printf("%s/%s", PACKAGE_PIXMAPS_DIR, pixmapfile);

  if (!g_file_exists (filename)) {
    g_error (_("Couldn't find file %s !"), filename);
  }

  smallnumbers_pixmap = gdk_imlib_load_image (filename);

  g_free (filename);
  
  return(smallnumbers_pixmap);
}

/**
 * Set the focus of the given image (highlight or not)
 *
 */
void set_image_focus(GdkImlibImage *pixmap, gboolean focus)
{
  GdkImlibColorModifier mod;

  gdk_imlib_get_image_modifier(pixmap,&mod);

  if(focus)
      mod.brightness=(double)BRIGHT;
  else
      mod.brightness=(double)BRIGHTLESS;

  gdk_imlib_set_image_modifier(pixmap,&mod);
}

/**
 * Callback over a canvas item, this function will highlight the focussed item
 *
 */
gint item_event_focus(GnomeCanvasItem *item, GdkEvent *event, GdkImlibImage *pixmap)
{

  switch (event->type) 
    {
    case GDK_ENTER_NOTIFY:
      set_image_focus(pixmap, TRUE);
      gnome_canvas_item_set (item,
			     "image", pixmap,
			     NULL);

      break;
    case GDK_LEAVE_NOTIFY:
      set_image_focus(pixmap, FALSE);
      gnome_canvas_item_set (item,
			     "image", pixmap,
			     NULL);

      break;
    default:
      break;
    }
  return FALSE;
}

/* Play a sound installed in the Gnome sound list */
void play_sound (const char *soundlistfile, const char *which)
{
  gchar *filename;

  filename = g_strdup_printf("%s/%s.wav", PACKAGE_SOUNDS_DIR, which);

  if (!g_file_exists (filename)) {
    g_error (_("Couldn't find file %s !"), filename);
  }

  if (properties->fx) {
    gnome_sound_play (filename);
  //    gnome_triggers_do (NULL, NULL, soundlistfile, which, NULL);
  }

  g_free (filename);
}


