/* gcompris - gcompris-board.h
 *
 * Time-stamp: <2001/01/22 23:19:29 bruno>
 *
 * Copyright (C) 2000 Bruno Coudoin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef TYPES_H
#define TYPES_H

#include <config.h>
#include <gmodule.h>
#include <gnome.h>



typedef struct _GcomprisGame         GcomprisGame;
typedef struct _GcomprisCave         GcomprisCave;


/*****************************************************************************/
/* The following structure describes a game object.  */

typedef GcomprisGame* (*GcomprisGameNew)      ();
typedef void          (*GcomprisStartGame)    (GnomeCanvas *theCanvas, 
					       char *opt);
typedef void          (*GcomprisPauseGame)    (gboolean pause);
typedef void          (*GcomprisEndGame)      (void);
typedef void          (*GcomprisFreeGame)     (void);
typedef gint          (*GcomprisKeyPress)     (guint keyval);
typedef void          (*GcomprisOK)           (void);
typedef void          (*GcomprisSetLevel)     (guint level);


/*****************************************************************************/
/* GcomprisGame related declarations */


struct _GcomprisGame
{
  /* Name of this game */
  gchar               *name;
  gchar               *description;
  gchar               *icon_name;

  /* Size of the window in which we have to run */
  gint16               width;
  gint16               height;
  GnomeCanvas         *canvas;

  /* Pointors to game's own functions */
  GcomprisGameNew      new_game;
  GcomprisStartGame    start_game;

  GcomprisPauseGame    pause_game;
  GcomprisEndGame      end_game;
  GcomprisEndGame      free_game;
  GcomprisKeyPress     key_press;

  GcomprisOK           ok;
  GcomprisSetLevel     set_level;

  /* Dynamic game information (status) */  
  guint                level;
  guint                maxlevel;
  guint                sublevel;
  guint                number_of_sublevel;

};



/*****************************************************************************/
/* Gcompris global related declarations */


typedef enum
{
  GAME_FINISHED      = 1 << 0,
  GAME_RUNNING       = 1 << 1,
  GAME_PAUSING       = 1 << 2
} GameFlags;



/*****************************************************************************/
/* Some Global definitions */
#define BRIGHTLESS 220
#define BRIGHT     256



/* Method usefull for boards provided by gcompris */
GnomeCanvasItem *set_background(GnomeCanvasGroup *parent, gchar *file);
gint game_running (void);
void bar_set_level (GcomprisGame *gcomprisGame);
void bar_set_timer (guint time);
void bar_set_maxtimer (guint time);
void bar_set_message (const gchar *message);


#endif

/* Local Variables: */
/* mode:c */
/* eval:(load-library "time-stamp") */
/* eval:(make-local-variable 'write-file-hooks) */
/* eval:(add-hook 'write-file-hooks 'time-stamp) */
/* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
/* End: */
