/* gcompris - gcompris.c
 *
 * Time-stamp: <2001/04/11 22:56:18 bruno>
 *
 * Copyright (C) 2000 Bruno Coudoin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "gcompris.h"
#include "gcompris-board.h"

GtkWidget *window;
GtkWidget *drawing_area;
GnomeCanvas *canvas;
GnomeCanvas *canvas_bar;

static gint current_board=-1;

static void pause_game (void);
static void start_game (void);
static gint pause_game_cb (GtkWidget *widget, gpointer data);
static void quit_cb (GtkWidget *widget, gpointer data);
static void about_cb (GtkWidget *widget, gpointer data);
static void help_cb (GtkWidget *widget, gpointer data);
static gint end_game_box (void);
static gint game_widget_key_press_callback (GtkWidget   *widget,
					    GdkEventKey *event,
					    gpointer     client_data);


static GnomeUIInfo game_menu[] =
{
  GNOMEUIINFO_MENU_EXIT_ITEM (quit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[] = {
	GNOMEUIINFO_ITEM ("Help", NULL, help_cb, NULL),
	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
	GNOMEUIINFO_MENU_GAME_TREE (game_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

GcomprisProperties *properties;
GcomprisGame *currentGame;
GameFlags state = GAME_FINISHED;


/****************************************************************************/
/* Some constants.  */

/* This message will be printed in the statusline, if no message was
   specified in a game file.  */

static char *default_message= "GCompris (c) 2000 Bruno Coudoin (Released Under the GPL)";


static gint
game_widget_key_press_callback (GtkWidget   *widget,
				GdkEventKey *event,
				gpointer     client_data)
{

  switch (event->keyval)
    {
    case GDK_Escape:
      quit_cb(NULL, NULL);
      return TRUE;
    case GDK_KP_Enter:
    case GDK_Return:
      /* We make here a shortcup to OK 
	 Warning, a board that would need the enter key will never receive it */
      if (currentGame!=NULL && currentGame->ok)
	{
	  currentGame->ok ();
	}
      return TRUE;
    default:
      /* If the board need to receive key pressed */
      if (currentGame!=NULL && currentGame->key_press)
	{
	  return(currentGame->key_press (event->keyval));
	}
    }

  return FALSE;
};

GnomeCanvasItem *set_background(GnomeCanvasGroup *parent, gchar *file)
{
  gchar *filename;
  GdkImlibImage *background_pixmap = NULL;
  static GnomeCanvasItem *backgroundimg = NULL;
  double xratio, yratio;
  guint yminus = BARHEIGHT+30;
  guint xminus = 30;
  double boardwidth, boardheight;

  filename = g_strdup_printf("%s/%s", PACKAGE_PIXMAPS_DIR, file);

  if (!g_file_exists (filename)) {
    g_error (_("Couldn't find pixmap file %s/%s !"), PACKAGE_PIXMAPS_DIR, file);
  }

  background_pixmap = gdk_imlib_load_image (filename);
  
  g_free (filename);

  yratio=(gdk_screen_height()-yminus)/(float)background_pixmap->rgb_height;
  xratio=(gdk_screen_width()-xminus)/(float)background_pixmap->rgb_width;
  printf("The gdk_screen_width()=%f gdk_screen_height()=%f\n", 
	 (double)gdk_screen_width(), (double)gdk_screen_height());
  printf("The xratio=%f yratio=%f\n", xratio, yratio);

  yratio=xratio=MIN(xratio, yratio);

  /* The canvas does not look pretty when resized above 1 ratio. Avoid that */
  xratio=MIN(1.0, xratio);

  //  printf("Calculated x ratio xratio=%f BOARDWIDTH*xratio=%f\n", 
  //          xratio, background_pixmap->rgb_width*xratio);
  boardheight= background_pixmap->rgb_height;
  boardwidth= background_pixmap->rgb_width;

  gnome_canvas_set_pixels_per_unit (canvas, xratio);

  backgroundimg=gnome_canvas_item_new (parent,
				       gnome_canvas_image_get_type (),
				       "image", background_pixmap, 
				       "x", 0.0,
				       "y", 0.0,
				       "width", (double) boardwidth,
				       "height", (double) boardheight,
				       "anchor", GTK_ANCHOR_NW,
				       NULL);

  gnome_canvas_set_scroll_region (canvas,
				  0, 0,
				  boardwidth,
				  boardheight);

  gtk_widget_set_usize (GTK_WIDGET(canvas), boardwidth*xratio, boardheight*xratio);

  /* Create the spot for the bar */
  gnome_canvas_set_pixels_per_unit (canvas_bar, xratio);
  gnome_canvas_set_scroll_region (canvas_bar,
				  0, 0,
				  boardwidth,
				  BARHEIGHT);
  gtk_widget_set_usize (GTK_WIDGET(canvas_bar),  boardwidth*xratio,  BARHEIGHT*xratio);

  return (backgroundimg);
}

static void setup_window ()
{
  GtkWidget *vbox;
  GtkWidget *statusbar= NULL;
  /*  GdkCursor *hand_cursor; */

  window = gnome_app_new (PACKAGE, _("GCompris I Have Understood"));
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);

  gtk_widget_realize (window);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (quit_cb), NULL);
  gtk_signal_connect (GTK_OBJECT (window), "key_press_event",
		      GTK_SIGNAL_FUNC (game_widget_key_press_callback), 0);


  /*  hand_cursor = gdk_cursor_new(GDK_HAND2);
      gdk_window_set_cursor	 (window->window, hand_cursor);
      gdk_cursor_destroy(hand_cursor);
  */

  /* -------------- */

  /* For non anti alias canvas */
  gtk_widget_push_visual (gdk_imlib_get_visual ());
  gtk_widget_push_colormap (gdk_imlib_get_colormap ());

  /* For anti alias canvas */
  /*
  gtk_widget_push_visual(gdk_rgb_get_visual());
  gtk_widget_push_colormap(gdk_rgb_get_cmap());
  */

  /* For non anti alias canvas */
  canvas     = GNOME_CANVAS(gnome_canvas_new ());
  canvas_bar = GNOME_CANVAS(gnome_canvas_new ());

  /* For anti alias canvas */
  /*
  canvas     = GNOME_CANVAS(gnome_canvas_new_aa ());
  canvas_bar = GNOME_CANVAS(gnome_canvas_new_aa ());
  */

  /* Create a vertical box in which I put first the play board area, then the button bar */
  vbox = gtk_vbox_new (FALSE, 0);

  gnome_app_set_contents (GNOME_APP (window), GTK_WIDGET(vbox));

  gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(canvas), TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(canvas_bar), TRUE, TRUE, 0);

  gtk_widget_pop_colormap ();
  gtk_widget_pop_visual ();


  gtk_widget_show (GTK_WIDGET(vbox));
  gtk_widget_show (GTK_WIDGET(canvas));
  gtk_widget_show (GTK_WIDGET(canvas_bar));
  
  /* Run the menu */
  properties->menu->start_game (canvas, NULL);
  currentGame = properties->menu;

  /* Create the status bar */
  statusbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_USER);

  /* Run the bar */
  bar_start(canvas_bar, statusbar);

  /* Menu creation */  
  gnome_app_create_menus (GNOME_APP (window), main_menu);
  
  gnome_app_set_statusbar (GNOME_APP (window), statusbar);
  gnome_appbar_set_default (GNOME_APPBAR (statusbar), _(default_message));

  gnome_app_install_menu_hints (GNOME_APP (window), main_menu);
  
}

/* It implements gcompris's own way to load help in order to be
 * Package relocatable.
 * Unfortunatly, this does not supports I18N
 */
static void help_cb (GtkWidget *widget, gpointer data)
{
  gnome_help_goto(NULL, PACKAGE_HELP_DIR "/C/gcompris.html");
}

static void about_cb (GtkWidget *widget, gpointer data)
{
  static GtkWidget *about;


  const gchar *authors[] = {
    "Bruno Coudoin <bruno.coudoin@free.fr>",
    NULL
  };

  if (about != NULL) {
    gdk_window_raise (about->window);
    gdk_window_show (about->window);
    return;
  }

  about = gnome_about_new ("GCompris", VERSION,
                        ("Copyright 2000 Bruno Coudoin (Released under the GPL)"),
                        authors,
                        _("A simple education game for children starting at 3."),
                        PACKAGE_PIXMAPS_DIR "/gcompris-about.jpg");

  gtk_object_set_data (GTK_OBJECT (about), "about", about);
  gtk_window_set_modal (GTK_WINDOW (about), TRUE);

  gtk_signal_connect (GTK_OBJECT (about), "destroy", GTK_SIGNAL_FUNC
		      (gtk_widget_destroyed), &about);
  gnome_dialog_set_parent (GNOME_DIALOG (about), GTK_WINDOW (window));
  
  gtk_widget_show (about);
}

/*
 * Confirmation dialog
 */
static gint end_game_box ()
{
  GameFlags pause_state;
  static GtkWidget *box;
  gint status;

  if (box)
    return 0;

  pause_state=state;
  if (pause_state==GAME_RUNNING)
    pause_game_cb (NULL, (gpointer) 0);
  
  box = gnome_message_box_new (
			       _("Do you really want to end this game?"),
			       GNOME_MESSAGE_BOX_QUESTION,
			       GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
			       NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (box), GTK_WINDOW
			   (window));
  gnome_dialog_set_default (GNOME_DIALOG (box), 0);
  status = gnome_dialog_run (GNOME_DIALOG (box));
  box = NULL;
  if (pause_state==GAME_RUNNING)
    pause_game_cb (NULL, (gpointer) 0);
  
  return (status);
}

static void cleanup()
{
  current_board=-1;
  currentGame=NULL;

  properties->menu->end_game ();

}

gint game_running ()
{
  return (state==GAME_RUNNING);
}


static gint pause_game_cb (GtkWidget *widget, gpointer data)
{
  if(game_running()) {
    pause_game();
  } else {
    start_game();
  }
  return(TRUE);
}

static void pause_game()
{
  if (currentGame!=NULL && currentGame->pause_game)
    {
      currentGame->pause_game (TRUE);
    }
  state=GAME_PAUSING;
}

static void start_game()
{

  if(state==GAME_PAUSING)
    {
      /* If paused, then start the same game again */
      if (currentGame!=NULL && currentGame->pause_game)
	{
	  currentGame->pause_game (FALSE);
	}
    }
  state=GAME_RUNNING;
}

void end_game()
{
  state=GAME_FINISHED;

  if (currentGame!=NULL && currentGame->end_game)
    {
      currentGame->end_game ();
    }

  if (currentGame == properties->menu)
    {
      /* We are in the menu leave GCompris */
      quit_cb(NULL, NULL);
    }

  /* Run the menu */
  properties->menu->start_game (canvas, NULL);
  currentGame = properties->menu;
}

static void quit_cb (GtkWidget *widget, gpointer data)
{
  if (end_game_box ())
    return;

  cleanup();

  gtk_main_quit ();

}


GcomprisProperties *get_properties ()
{
  return (properties);
}

static void load_properties ()
{
  properties = gcompris_properties_new ();
}

/**
 * Inform the main that a game is started
 */
void set_running_game(GcomprisGame *game) 
{
  currentGame=game;
  state=GAME_RUNNING;
}

/**
 * Inform the main that a game is started
 */
GcomprisGame *get_running_game() 
{
  return currentGame;
}

int
main (int argc, char *argv[])
{

  srand (time (NULL));

  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);

  gnome_init ("GCompris", VERSION, argc, argv);

  load_properties ();

  setup_window ();

  gtk_widget_show (window);

  gtk_main ();
  return 0;
}


/* Local Variables: */
/* mode:c */
/* eval:(load-library "time-stamp") */
/* eval:(make-local-variable 'write-file-hooks) */
/* eval:(add-hook 'write-file-hooks 'time-stamp) */
/* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
/* End: */
