/* gcompris - clickgame.c
 *
 * Time-stamp: <2001/04/11 22:49:44 bruno>
 *
 * Copyright (C) 2000 Bruno Coudoin
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * The menu display icons of each games and let the user select one
 *
 * Seen from the main, the menu is a game like another.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "gcompris.h"
#include "gcompris-board.h"
#include "properties.h"
#include "gameutil.h"
#include "games.h"

#define SOUNDLISTFILE PACKAGE
#define MENU_PER_LINE 5

static GList *item_list = NULL;

static GcomprisGame *gcomprisGame = NULL;

/* Hash table of all displayed images  */
static GHashTable *menu_table= NULL;

static GnomeCanvasItem *menu_create_item(GnomeCanvasGroup *parent, GcomprisGame *game);
static gboolean next_spot();
static void create_info_area(GnomeCanvasGroup *parent);
static gint item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
static void display_game_icon(GcomprisGame *game);

static double current_x = 0.0;
static double current_y = 0.0;

static GList		*gamelist;	/* List of Game */

/* Information items */
static GnomeCanvasItem *gamename_item, *description_item;

typedef struct {
  GcomprisGame *game;
  GnomeCanvasItem *item;
  GdkImlibImage *image;
} MenuItem;


/*
 * Main entry point mandatory for each Gcompris's game
 * ---------------------------------------------------
 *
 */


/*
 * in : boolean TRUE = PAUSE : FALSE = CONTINUE
 *
 */
static void menu_pause_game (gboolean pause)
{

  if(gcomprisGame==NULL)
    return;

}

/*
 * opt             : unused
 */
static void menu_start_game (GnomeCanvas *theCanvas,char *opt)
{
  current_x = 0.0;
  current_y = 0.0;

  if(gcomprisGame!=NULL)
    {
      gcomprisGame->canvas = theCanvas;

      set_background(gnome_canvas_root(gcomprisGame->canvas), "gcompris-init.jpg");

      set_game_size(theCanvas, gcomprisGame);

      g_list_foreach (gamelist, (GFunc) display_game_icon, NULL);
      
      create_info_area(gnome_canvas_root(gcomprisGame->canvas));
      
      menu_pause_game(FALSE);

    }

}

static void
menu_end_game ()
{

  if(gcomprisGame!=NULL)
    {
      menu_pause_game(TRUE);
    }
}

static void
menu_free_game ()
{

  if(gcomprisGame!=NULL)
    {
      menu_pause_game(TRUE);
      g_free (gcomprisGame);
      gcomprisGame=NULL;
    }
}

/*
 */
GcomprisGame *
menu_game_new ()
{

  /* This is a singleton */
  if(gcomprisGame!=NULL)
    return gcomprisGame;

  gcomprisGame= g_malloc (sizeof (GcomprisGame));

  if (gcomprisGame)
    {
      GList		*gamelistentry;	/* List of Entry points for games */
      guint i;

      gcomprisGame->name=            _("Main Menu");
      gcomprisGame->description=     _("Select a Game");

      gcomprisGame->pause_game=&menu_pause_game;
      gcomprisGame->start_game=&menu_start_game;
      gcomprisGame->end_game=&menu_end_game;
      gcomprisGame->free_game=&menu_free_game;
      gcomprisGame->key_press=NULL;
      gcomprisGame->ok=NULL;
      gcomprisGame->set_level=NULL;

      /* Load all games entry point in a list */
      gamelistentry = NULL;
      
      /* ADD YOUR GAMES IN THE LIST BELOW */
      gamelistentry = g_list_append (gamelistentry, &clickgame_game_new);
      gamelistentry = g_list_append (gamelistentry, &gletters_game_new);
      gamelistentry = g_list_append (gamelistentry, &smallnumbers_game_new);
      gamelistentry = g_list_append (gamelistentry, &wordsgame_game_new);
      gamelistentry = g_list_append (gamelistentry, &algebra_game_new);
      gamelistentry = g_list_append (gamelistentry, &clockgame_game_new);
      gamelistentry = g_list_append (gamelistentry, &shapegame_game_new);
      gamelistentry = g_list_append (gamelistentry, &planegame_game_new);
      
      /*---------------------------------*/
      
      /* Based on the entry points, load all the games now */
      gamelist=NULL;
      
      for(i=0; i<g_list_length(gamelistentry); i++)
	{
	  GcomprisGameNew newgame;
	  newgame = g_list_nth_data(gamelistentry, i);
	  gamelist = g_list_append (gamelist, 
				    newgame());
	}      
    }

  return gcomprisGame;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

static void display_game_icon(GcomprisGame *game)
{
  if (game!=NULL)
    {
      menu_create_item(gnome_canvas_root(gcomprisGame->canvas), game);
    }
}

/*
 * Calculate the next stop where to place an item
 * return false if there is no more space left
 */
static gboolean next_spot() 
{
  if(current_x==0.0)
    {
      /* Initialisation case */
      current_x = gcomprisGame->width/MENU_PER_LINE;
      current_y = gcomprisGame->height/MENU_PER_LINE;
      return(TRUE);
    }

  current_x += gcomprisGame->width/MENU_PER_LINE;
  if(current_x>=gcomprisGame->width-100)
    {
      current_x = gcomprisGame->width/MENU_PER_LINE;
      current_y += 100;
    }

  return(TRUE);
}

static GnomeCanvasItem *menu_create_item(GnomeCanvasGroup *parent, GcomprisGame *game)
{
  GdkImlibImage *menu_pixmap = NULL;
  GnomeCanvasItem *item;
  MenuItem *menuitem;

  menuitem = malloc(sizeof(MenuItem));

  menu_pixmap = load_pixmap(game->icon_name);
  next_spot();
  item = gnome_canvas_item_new (parent,
				gnome_canvas_image_get_type (),
				"image", menu_pixmap, 
				"x", (double)current_x,
				"y", (double)current_y,
				"width", (double) menu_pixmap->rgb_width,
				"height", (double) menu_pixmap->rgb_height,
				"anchor", GTK_ANCHOR_CENTER,
				NULL);
  set_image_focus(menu_pixmap, FALSE);
  item_list = g_list_append (item_list, item);

  if (!menu_table)
    {
      menu_table= g_hash_table_new (g_direct_hash, g_direct_equal);
    }

  menuitem->game=game;
  menuitem->item=item;
  menuitem->image=menu_pixmap;
  g_hash_table_insert (menu_table, item, menuitem);

  gtk_signal_connect(GTK_OBJECT(item), "event",
		     (GtkSignalFunc) item_event,
		     NULL);

  gtk_signal_connect(GTK_OBJECT(item), "event",
		     (GtkSignalFunc) item_event_focus,
		     menu_pixmap);

  return (item);
}

static gint
item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
{
  MenuItem *menuitem;

  menuitem = (MenuItem *)g_hash_table_lookup (menu_table, item);

  switch (event->type) 
    {
    case GDK_ENTER_NOTIFY:
      gnome_canvas_item_set (gamename_item,
			     "text", menuitem->game->name,
			     NULL);
      gnome_canvas_item_set (description_item,
			     "text",  menuitem->game->description,
			     NULL);

      break;
    case GDK_LEAVE_NOTIFY:
      gnome_canvas_item_set (gamename_item,
			     "text", " ",
			     NULL);

      gnome_canvas_item_set (description_item,
			     "text",  " ",
			     NULL);

      break;
    case GDK_BUTTON_PRESS:
      play_sound (SOUNDLISTFILE, "gobble");
      set_running_game(menuitem->game);
      menuitem->game->start_game (gcomprisGame->canvas, NULL);
      break;
      
    default:
      break;
    }
  return FALSE;
}

static void create_info_area(GnomeCanvasGroup *parent)
{
  GnomeCanvasItem *rootitem, *rectitem;
  GdkFont *gdk_font, *gdk_font_small;

  /* Load a gdk font */
  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-180-*-*-*-*-*-*");
  gdk_font_small = gdk_font_load ("-adobe-times-medium-r-normal--*-120-*-*-*-*-*-*");

  rootitem = \
    gnome_canvas_item_new (parent,
			   gnome_canvas_group_get_type (),
			   "x", (double)gcomprisGame->width/2,
			   "y", (double)(gcomprisGame->height*13)/20,
			   NULL);

  rectitem = \
    gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
			   gnome_canvas_rect_get_type (),
			   "x1", (double)-gcomprisGame->width/2+20,
			   "y1", (double)0,
			   "x2", (double)gcomprisGame->width/2-20,
			   "y2", (double)(gcomprisGame->height*2)/10,
			   "fill_color", "black",
			   "outline_color", "blue",
			   "width_units", (double)5,
			   NULL);


  gamename_item = \
    gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
			   gnome_canvas_text_get_type (),
			   "text", "",
			   "font_gdk", gdk_font,
			   "x", (double) 0,
			   "y", (double) 25,
			   "anchor", GTK_ANCHOR_CENTER,
			   "fill_color", "green",
			   NULL);

  description_item = \
    gnome_canvas_item_new (GNOME_CANVAS_GROUP(rootitem),
			   gnome_canvas_text_get_type (),
			   "text", "",
			   "font_gdk", gdk_font_small,
			   "x", (double) 0,
			   "y", (double) 60,
			   "anchor", GTK_ANCHOR_NORTH,
			   "fill_color", "lightgreen",
			   "justification", GTK_JUSTIFY_CENTER,
			   NULL);
}





/* Local Variables: */
/* mode:c */
/* eval:(load-library "time-stamp") */
/* eval:(make-local-variable 'write-file-hooks) */
/* eval:(add-hook 'write-file-hooks 'time-stamp) */
/* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
/* End: */
