/* gcompris - smallnumbers.c
 *
 * Time-stamp: <2001/03/26 01:09:19 bruno>
 *
 * Copyright (C) 2000 Bruno Coudoin
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "gcompris-board.h"
#include "properties.h"
#include "gameutil.h"

#define SOUNDLISTFILE PACKAGE

static GList *item_list = NULL;
static GList *item2del_list = NULL;

static GcomprisGame *gcomprisGame = NULL;

static gint dummy_id = 0;
static gint drop_items_id = 0;

static char *numbers = "123456";

/* Hash table of all displayed letters  */
static GHashTable *letters_table= NULL;

static GnomeCanvasItem *smallnumbers_create_item(GnomeCanvasGroup *parent);
static gint smallnumbers_drop_items (GtkWidget *widget, gpointer data);
static gint smallnumbers_move_items (GtkWidget *widget, gpointer data);
static void smallnumbers_destroy_item(GnomeCanvasItem *item);
static void smallnumbers_destroy_items(void);
static void smallnumbers_destroy_all_items(void);
static void smallnumbers_next_level(void);
static void smallnumbers_add_new_item(void);
gint smallnumbers_key_press(guint keyval);

static void player_win(GnomeCanvasItem *item);
static void player_loose(void);
static GnomeCanvasItem *item_find_by_title (const gchar *title);
static char *key_find_by_item (const GnomeCanvasItem *item);

static  guint32              fallSpeed = 0;
static  double               speed = 0.0;
static  double               imageZoom = 0.0;

/*
 * Main entry point mandatory for each Gcompris's game
 * ---------------------------------------------------
 *
 */

/*
 * in : boolean TRUE = PAUSE : FALSE = UNPAUSE
 *
 */
static void smallnumbers_pause_game (gboolean pause)
{

  if(gcomprisGame==NULL)
    return;

  if(pause)
    {
      if (dummy_id) {
	gtk_timeout_remove (dummy_id);
	dummy_id = 0;
      }
      if (drop_items_id) {
	gtk_timeout_remove (drop_items_id);
	drop_items_id = 0;
      }
    }
  else
    {
      if(!drop_items_id) {
	drop_items_id = gtk_timeout_add (1000,
					 (GtkFunction) smallnumbers_drop_items, NULL);
      }
      if(!dummy_id) {
	dummy_id = gtk_timeout_add (1000, (GtkFunction) smallnumbers_move_items, NULL);
      }
    }
}

/*
 * theCanvas       : the canvas object on which this board can play
 * opt             : optional param nonused here yet
 */
static void smallnumbers_start_game (GnomeCanvas *theCanvas, char *opt)
{

  gcomprisGame->canvas = theCanvas;

  if(gcomprisGame!=NULL)
    {
      set_background(gnome_canvas_root(gcomprisGame->canvas), "gcompris-bg.jpg");

      set_game_size(theCanvas, gcomprisGame);

      smallnumbers_next_level();

      smallnumbers_pause_game(FALSE);
    }
}

static void
smallnumbers_end_game ()
{
  if(gcomprisGame!=NULL)
    {
      smallnumbers_pause_game(TRUE);
      smallnumbers_destroy_all_items();
    }
}

static void
smallnumbers_free_game ()
{
  if(gcomprisGame!=NULL)
    {
      smallnumbers_pause_game(TRUE);
      smallnumbers_destroy_all_items();
      g_free (gcomprisGame);
      gcomprisGame=NULL;
    }
}

static void
smallnumbers_set_level (guint level)
{

  if(gcomprisGame!=NULL)
    {
      gcomprisGame->level=level;
      smallnumbers_next_level();
    }
}
/*
 */
GcomprisGame *
smallnumbers_game_new ()
{
  /* This is a singleton */
  if(gcomprisGame!=NULL)
    return gcomprisGame;

  gcomprisGame= g_malloc (sizeof (GcomprisGame));
  
  if (gcomprisGame)
    {
      gcomprisGame->name=            _("Numbers With Dices");
      gcomprisGame->description=     _("Type the keyboard number 1 to 6 before the dices reach the ground");
      gcomprisGame->icon_name="gameicons/smallnumbers.png";
      gcomprisGame->level = 1;
      gcomprisGame->maxlevel = 6;

      gcomprisGame->pause_game=&smallnumbers_pause_game;
      gcomprisGame->start_game=&smallnumbers_start_game;
      gcomprisGame->end_game=&smallnumbers_end_game;
      gcomprisGame->free_game=&smallnumbers_free_game;
      gcomprisGame->key_press=&smallnumbers_key_press;
      gcomprisGame->ok=NULL;
      gcomprisGame->set_level=	&smallnumbers_set_level;      
    }

  return gcomprisGame;
}

gint smallnumbers_key_press(guint keyval)
{
  gchar *old_value;
  gchar *old_name;
  char str[2];

   if(!game_running())
     return FALSE;

  /* Add some filter for control and shift key */
  switch (keyval)
    {
    case GDK_Shift_L:
    case GDK_Shift_R:
    case GDK_Control_L:
    case GDK_Control_R:
    case GDK_Caps_Lock:
    case GDK_Shift_Lock:
    case GDK_Meta_L:
    case GDK_Meta_R:
    case GDK_Alt_L:
    case GDK_Alt_R:
    case GDK_Super_L:
    case GDK_Super_R:
    case GDK_Hyper_L:
    case GDK_Hyper_R:
    case GDK_Mode_switch:
    case GDK_dead_circumflex:
    case GDK_Num_Lock:
      return FALSE; 
    case GDK_KP_0:
    case GDK_KP_Insert:
      keyval=GDK_0;
      break;
    case GDK_KP_1:
    case GDK_KP_End:
      keyval=GDK_1;
      break;
    case GDK_KP_2:
    case GDK_KP_Down:
      keyval=GDK_2;
      break;
    case GDK_KP_3:
    case GDK_KP_Page_Down:
      keyval=GDK_3;
      break;
    case GDK_KP_4:
    case GDK_KP_Left:
      keyval=GDK_4;
      break;
    case GDK_KP_5:
    case GDK_KP_Begin:
      keyval=GDK_5;
      break;
    case GDK_KP_6:
    case GDK_KP_Right:
      keyval=GDK_6;
      break;
    case GDK_KP_7:
    case GDK_KP_Home:
      keyval=GDK_7;
      break;
    case GDK_KP_8:
    case GDK_KP_Up:
      keyval=GDK_8;
      break;
    case GDK_KP_9:
    case GDK_KP_Page_Up:
      keyval=GDK_9;
      break;
    }

  sprintf(str, "%c", keyval);
  if (g_hash_table_lookup_extended (letters_table,
				    str,
				    (gpointer) &old_name,
				    (gpointer) &old_value))
    {

      player_win(item_find_by_title(str));

    }
  else
    {
      player_loose();
    }

  return TRUE;
}

/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/* set initial values for the next level */
static void smallnumbers_next_level() 
{

  bar_set_level(gcomprisGame);

  smallnumbers_destroy_all_items();

  /* Try the next level */
  speed=10+(40/gcomprisGame->level);
  fallSpeed=5000-gcomprisGame->level*200;
  gcomprisGame->number_of_sublevel=10;
  imageZoom=0.9+(0.5/gcomprisGame->level);
  bar_set_maxtimer(gcomprisGame->number_of_sublevel);
  gcomprisGame->sublevel=0;
  bar_set_timer(gcomprisGame->sublevel);
}


static void smallnumbers_move_item(GnomeCanvasItem *item)
{
  double x1, y1, x2, y2;

  gnome_canvas_item_move(item, 0, 2.0);

  gnome_canvas_item_get_bounds    (item,
				   &x1,
				   &y1,
				   &x2,
				   &y2);
  
  if(y1>gcomprisGame->height) {
    item2del_list = g_list_append (item2del_list, item);
    player_loose();
  }
}

static void smallnumbers_destroy_item(GnomeCanvasItem *item)
{
  char *key;

  key = key_find_by_item(item);
  /* Remove old letter  */
  g_hash_table_remove (letters_table, key);
  g_free (key);

  item_list = g_list_remove (item_list, item);
  item2del_list = g_list_remove (item2del_list, item);
  gtk_object_destroy (GTK_OBJECT(item));
}

/* Destroy items that falls out of the canvas */
static void smallnumbers_destroy_items()
{
  GnomeCanvasItem *item;

  while(g_list_length(item2del_list)>0) 
    {
      item = g_list_nth_data(item2del_list, 0);
      smallnumbers_destroy_item(item);
    }
}

/* Destroy all the items */
static void smallnumbers_destroy_all_items()
{
  GnomeCanvasItem *item;

  if(item_list)
    while(g_list_length(item_list)>0) 
      {
	item = g_list_nth_data(item_list, 0);
	smallnumbers_destroy_item(item);
      }

  /* Delete the letters_table */
  if(letters_table) {
    g_hash_table_destroy (letters_table);
    letters_table=NULL;
  }
}

/*
 * This does the moves of the game items on the play canvas
 *
 */
static gint smallnumbers_move_items (GtkWidget *widget, gpointer data)
{
  g_list_foreach (item_list, (GFunc) smallnumbers_move_item, NULL);

  /* Destroy iterm that falls out of the canvas */
  smallnumbers_destroy_items();

  dummy_id = gtk_timeout_add (speed, 
			      (GtkFunction) smallnumbers_move_items, NULL);

  return(FALSE);
}

static GnomeCanvasItem *smallnumbers_create_item(GnomeCanvasGroup *parent)
{
  GdkImlibImage *smallnumbers_pixmap = NULL;
  GnomeCanvasItem *item;
  char *str;
  int i;
  char *lettersItem;

  if (!letters_table)
    {
      letters_table= g_hash_table_new (g_str_hash, g_str_equal);
    }

  lettersItem = g_malloc (2);

  /* Beware, since we put the letters in a hash table, we do not allow the same
     letter to be displayed two times */
  do {
    i=rand()%strlen(numbers);
    sprintf(lettersItem, "%c", numbers[i]);
  } while(item_find_by_title(lettersItem)!=NULL);

  str = g_strdup_printf("dice/gnome-dice%c.png", numbers[i]);

  smallnumbers_pixmap = load_pixmap(str);
  g_free(str);

  item = gnome_canvas_item_new (parent,
				gnome_canvas_image_get_type (),
				"image", smallnumbers_pixmap, 
				"x", (double)(rand()%(gcomprisGame->width-
						      (guint)(smallnumbers_pixmap->rgb_width*
						       imageZoom))),
				"y", (double) -smallnumbers_pixmap->rgb_height*imageZoom,
				"width", (double) smallnumbers_pixmap->rgb_width*imageZoom,
				"height", (double) smallnumbers_pixmap->rgb_height*imageZoom,
				"anchor", GTK_ANCHOR_NW,
				NULL);
  item_list = g_list_append (item_list, item);

  /* Add letter to hash table of all falling letters. */
  g_hash_table_insert (letters_table, lettersItem, item);

  return (item);
}

static void smallnumbers_add_new_item() 
{
  smallnumbers_create_item(gnome_canvas_root(gcomprisGame->canvas));
}

/*
 * This is called on a low frequency and is used to drop new items
 *
 */
static gint smallnumbers_drop_items (GtkWidget *widget, gpointer data)
{
  smallnumbers_add_new_item();

  drop_items_id = gtk_timeout_add (fallSpeed,
				   (GtkFunction) smallnumbers_drop_items, NULL);
  return (FALSE);
}

static void player_win(GnomeCanvasItem *item)
{
  smallnumbers_destroy_item(item);
  play_sound (SOUNDLISTFILE, "gobble");

  gcomprisGame->sublevel++;
  bar_set_timer(gcomprisGame->sublevel);

  if(gcomprisGame->sublevel>=gcomprisGame->number_of_sublevel) 
    {
      /* Try the next level */
      gcomprisGame->level++;
      if(gcomprisGame->level>gcomprisGame->maxlevel)
	gcomprisGame->level=gcomprisGame->maxlevel;
      smallnumbers_next_level();
      play_sound (SOUNDLISTFILE, "bonus");
    }
  else
    {
      /* Drop a new item now to speed up the game */
      if(g_list_length(item_list)==0)
	{
	  if (drop_items_id) {
	    /* Remove pending new item creation to sync the falls */
	    gtk_timeout_remove (drop_items_id);
	    drop_items_id = 0;
	  }
	  if(!drop_items_id) {
	    drop_items_id = gtk_timeout_add (0,
					     (GtkFunction) smallnumbers_drop_items, NULL);
	  }
	}
    }
}

static void player_loose()
{
  play_sound (SOUNDLISTFILE, "crash");
}


/* Return in item the key if the value equals the item */
static void get_item(char *key, char *value, char **item)
{
  if(value==*item) {
    *item=key;
  }
}

static char *
key_find_by_item (const GnomeCanvasItem *item)
{
  static char *key_found;

  key_found = (char *)item;

  if (!letters_table)
    return NULL;
  
  /* We have to loop to find this item's key */
  g_hash_table_foreach (letters_table,
			(GHFunc) get_item,
			&key_found);

  return key_found;
}

static GnomeCanvasItem *
item_find_by_title (const gchar *title)
{
  if (!letters_table)
    return NULL;
  
  return g_hash_table_lookup (letters_table, title);
}



/* Local Variables: */
/* mode:c */
/* eval:(load-library "time-stamp") */
/* eval:(make-local-variable 'write-file-hooks) */
/* eval:(add-hook 'write-file-hooks 'time-stamp) */
/* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
/* End: */
